<?php
/**
 * Custom Post Type for Wizard Forms and Taxonomies.
 */
class WordPress_Form_Wizard_Questions extends WordPress_Form_Wizard
{
    protected $plugin_name;
    protected $version;
    protected $prefix;

    /**
     * Constructor.
     *
     * @author Daniel Barenkamp
     *
     * @version 1.0.0
     *
     * @since   1.0.0
     * @link    http://www.welaunch.io
     *
     * @param string $plugin_name
     * @param string $version
     */
    public function __construct($plugin_name, $version)
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        
        $this->prefix = "wordpress_form_wizard_";
    }

    /**
     * Init.
     *
     * @author Daniel Barenkamp
     *
     * @version 1.0.0
     *
     * @since   1.0.0
     * @link    http://www.welaunch.io
     *
     * @return bool
     */
    public function init()
    {
        global $wordpress_form_wizard_options;
        $this->options = $wordpress_form_wizard_options;

        if (!$this->get_option('enable')) {
            return false;
        }

        $transient_name = $this->prefix . 'meta_keys';
        $this->post_meta_keys = get_transient( $transient_name );
        // if ( false === $this->post_meta_keys ) { 

            global $wpdb;
            $sql = "SELECT DISTINCT meta_key
                            FROM " . $wpdb->postmeta . "
                            ORDER BY meta_key ASC";

            $postMetaKeys = $wpdb->get_results( $sql, 'ARRAY_A' );
            $this->post_meta_keys = array();
            if(!empty($postMetaKeys)) {
                foreach($postMetaKeys as $postMetaKey) {
                    if(!isset($postMetaKey['meta_key']) || empty($postMetaKey['meta_key']) ) {
                        continue;
                    }
                    $this->post_meta_keys[$postMetaKey['meta_key']] = $postMetaKey['meta_key'];
                }
            }
            
            set_transient( $transient_name, $this->post_meta_keys, WEEK_IN_SECONDS);
        // }

        $this->register_form_wizard_post_type();

        add_action('wp_ajax_wordpress_form_wizard_add_question', array($this, 'add_question') );
        add_action('wp_ajax_wordpress_form_wizard_show_question', array($this, 'show_question') );
        add_action('wp_ajax_wordpress_form_wizard_hide_question', array($this, 'hide_question') );
        add_action('wp_ajax_wordpress_form_wizard_sort_questions', array($this, 'sort_questions') );
        add_action('wp_ajax_wordpress_form_wizard_get_terms', array($this, 'get_terms') );
    }

    public function get_terms()
    {
       $response = array(
            'status' => 0,
            'message' => '',
            'data' => array()
        );

        if(!current_user_can('edit_posts') ) {
            $response['message'] = esc_html__('No Access rights', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            $response['message'] = esc_html__('No AJAX call', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['taxonomy']) || empty($_POST['taxonomy'])) {
            $response['message'] = esc_html__('Question id missing.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }
        $taxonomy = esc_html( $_POST['taxonomy'] );

         $terms = get_terms( array(
            'taxonomy' => $taxonomy,
            'hide_empty' => false,
            'fields' => 'id=>name',
        ));

        if (empty($terms)) {
            $response['message'] = esc_html__('No terms found.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }


        $response['status'] = 1;
        $response['message'] = esc_html__('terms found.', 'wordpress-form-wizard');
        $response['data'] = $terms;

        echo json_encode($response);
        die();

    }
    /**
     * Save Points on Question Answered
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https:/welaunch.io
     * @return  [type]             [description]
     */
    public function show_question()
    {
        $response = array(
            'status' => 0,
            'message' => '',
            'data' => array()
        );

        if(!current_user_can('edit_posts') ) {
            $response['message'] = esc_html__('No Access rights', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            $response['message'] = esc_html__('No AJAX call', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['question']) || empty($_POST['question'])) {
            $response['message'] = esc_html__('No Question.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        $question = (int) $_POST['question'];
        wp_publish_post( $question );

        $response['status'] = 1;
        $response['message'] = esc_html__('Question moved to Trash.', 'wordpress-form-wizard');
        $response['data'] = array(
            'id' => $question,
        );

        echo json_encode($response);
        die();
    }

/**
     * Save Points on Question Answered
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https:/welaunch.io
     * @return  [type]             [description]
     */
    public function hide_question()
    {
        $response = array(
            'status' => 0,
            'message' => '',
            'data' => array()
        );

        if(!current_user_can('edit_posts') ) {
            $response['message'] = esc_html__('No Access rights', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            $response['message'] = esc_html__('No AJAX call', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['question']) || empty($_POST['question'])) {
            $response['message'] = esc_html__('No Question.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        $question = (int) $_POST['question'];
        wp_trash_post( $question );

        $response['status'] = 1;
        $response['message'] = esc_html__('Question moved to Trash.', 'wordpress-form-wizard');
        $response['data'] = array(
            'id' => $question,
        );

        echo json_encode($response);
        die();
    }

    /**
     * Save Points on Question Answered
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https:/welaunch.io
     * @return  [type]             [description]
     */
    public function add_question()
    {
        $response = array(
            'status' => 0,
            'message' => '',
            'data' => array()
        );

        if(!current_user_can('edit_posts') ) {
            $response['message'] = esc_html__('No Access rights', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            $response['message'] = esc_html__('No AJAX call', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['question']) || empty($_POST['question'])) {
            $response['message'] = esc_html__('No Question.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['postId']) || empty($_POST['postId'])) {
            $response['message'] = esc_html__('No Question.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        $question = $_POST['question'];
        $postId = $_POST['postId'];
        
        $questionId = wp_insert_post(array(
            'post_title' => $question,
            'post_type' => 'wizard_question',
            'post_parent' => $postId,
            'post_status' => 'publish',
        ));

        if (!$questionId) {
            $response['message'] = esc_html__('Question could not be created.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        $questions = get_post_meta($postId, $this->prefix . 'questions', true);

        if(!empty($questions) && is_array($questions)) {
            $questions[] = $questionId;
        } else {
            $questions = array(
                $questionId
            );
        }

        update_post_meta($postId, $this->prefix . 'questions', $questions);

        $response['status'] = 1;
        $response['message'] = esc_html__('Question created.', 'wordpress-form-wizard');
        $response['data'] = array(
            'id' => $questionId,
            'title' => $question,
        );

        echo json_encode($response);
        die();
    }

    /**
     * Save Points on Question Answered
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https:/welaunch.io
     * @return  [type]             [description]
     */
    public function sort_questions()
    {
        $response = array(
            'status' => 0,
            'message' => '',
            'data' => array()
        );

        if(!current_user_can('edit_posts') ) {
            $response['message'] = esc_html__('No Access rights', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            $response['message'] = esc_html__('No AJAX call', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['questionIds']) || empty($_POST['questionIds'])) {
            $response['message'] = esc_html__('Question Ids missing.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['postId']) || empty($_POST['postId'])) {
            $response['message'] = esc_html__('No Post Id.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        $questionIds = array_filter($_POST['questionIds'], 'is_numeric');
        $postId = $_POST['postId'];
        
        if (empty($questionIds)) {
            $response['message'] = esc_html__('Question Ids empty.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        update_post_meta($postId, $this->prefix . 'questions', $questionIds);
        $response['status'] = 1;
        $response['message'] = esc_html__('Question sorting updated.', 'wordpress-form-wizard');
        $response['data'] = array(
            'id' => $questionId
        );

        echo json_encode($response);
        die();
    }

    /**
     * Register Wizard Form Post Type.
     *
     * @author Daniel Barenkamp
     *
     * @version 1.0.0
     *
     * @since   1.0.0
     * @link    http://www.welaunch.io
     *
     * @return bool
     */
    public function register_form_wizard_post_type()
    {
        $singular = esc_html__('Wizard Question', 'wordpress-form-wizard');
        $plural = esc_html__('Wizard Questions', 'wordpress-form-wizard');

        $labels = array(
            'name' => esc_html__('Wizard Questions', 'wordpress-form-wizard'),
            'all_items' => $plural,
            'singular_name' => $singular,
            'add_new' => sprintf(__('New %s', 'wordpress-form-wizard'), $singular),
            'add_new_item' => sprintf(__('Add New %s', 'wordpress-form-wizard'), $singular),
            'edit_item' => sprintf(__('Edit %s', 'wordpress-form-wizard'), $singular),
            'new_item' => sprintf(__('New %s', 'wordpress-form-wizard'), $singular),
            'view_item' => sprintf(__('View %s', 'wordpress-form-wizard'), $plural),
            'search_items' => sprintf(__('Search %s', 'wordpress-form-wizard'), $plural),
            'not_found' => sprintf(__('No %s found', 'wordpress-form-wizard'), $plural),
            'not_found_in_trash' => sprintf(__('No %s found in trash', 'wordpress-form-wizard'), $plural),
        );

        $args = array(
            'labels' => $labels,
            'public' => false,
            'exclude_from_search' => false,
            'show_ui' => true,
            'menu_position' => 57,
            'query_var' => 'wizard-question',
            'supports' => array('title', 'author'),
            'menu_icon' => 'dashicons-welcome-learn-more',
        );

        register_post_type('wizard_question', $args);

    }


    /**
     * Add custom ticket metaboxes
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @param   [type]                       $post_type [description]
     * @param   [type]                       $post      [description]
     */
    public function add_custom_metaboxes($post_type, $post)
    {
        add_meta_box('wordpress-form_wizard-general', esc_html__('General', 'wordpress-form-wizard'), array($this, 'general'), 'wizard_question', 'normal', 'high');
        add_meta_box('wordpress-form_wizard-answers', esc_html__('Answers', 'wordpress-form-wizard'), array($this, 'answers'), 'wizard_question', 'normal', 'high');
    }

    /**
     * Display Metabox Address
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @return  [type]                       [description]
     */
    public function general()
    {
        global $post;

        wp_nonce_field(basename(__FILE__), 'wordpress_form_wizard_meta_nonce');

        if($this->is_new()) {
            $type = 'radio';
            $columns = '3';
            $style = "imge_top";
            $required = 'yes';
            $relation = 'OR';
            $dependencies = array();

            $views = 0;
            $completed = 0;
            
            $internal = "";
            $breadcrumb = "";
            $filtering = "filtering";
            $description = "";

            $slider_price = 1;
            $slider_suffix = '';
            $slider_min = 0;
            $slider_max = 1000;
            $slider_step = 1;
            $slider_value = 0;

        } else {
            $type = get_post_meta($post->ID, $this->prefix . 'type', true);
            $columns = get_post_meta($post->ID, $this->prefix . 'columns', true);
            $style = get_post_meta($post->ID, $this->prefix . 'style', true);
            $required = get_post_meta($post->ID, $this->prefix . 'required', true);
            $relation = get_post_meta($post->ID, $this->prefix . 'relation', true);
            $dependencies = get_post_meta($post->ID, $this->prefix . 'dependencies', true);

            $internal = get_post_meta($post->ID, $this->prefix . 'internal', true);
            $breadcrumb = get_post_meta($post->ID, $this->prefix . 'breadcrumb', true);
            $filtering = get_post_meta($post->ID, $this->prefix . 'filtering', true);
            $description = get_post_meta($post->ID, $this->prefix . 'description', true);

            $views = (int) get_post_meta($post->ID, $this->prefix . 'views', true);
            $completed = (int) get_post_meta($post->ID, $this->prefix . 'completed', true);

            $slider_price = (float) get_post_meta($post->ID, $this->prefix . 'slider_price', true);
            $slider_suffix = get_post_meta($post->ID, $this->prefix . 'slider_suffix', true);
            $slider_min = (int) get_post_meta($post->ID, $this->prefix . 'slider_min', true);
            $slider_max = (int) get_post_meta($post->ID, $this->prefix . 'slider_max', true);
            $slider_step = (int) get_post_meta($post->ID, $this->prefix . 'slider_step', true);
            $slider_value = (int) get_post_meta($post->ID, $this->prefix . 'slider_value', true);
        }

        $questions = array();
        $this->parentFormId = $post->post_parent;
        $backTo = "";
        if($this->parentFormId > 0) {

            $questions = get_post_meta($this->parentFormId, $this->prefix . 'questions', true);
            if(!empty($questions)) {
                $questions = get_posts(array(
                    'post_type' => 'wizard_question',
                    'post__in' => $questions,
                    'post_status' => 'any',
                    'orderby' => 'post__in',
                    'numberposts' => '-1',
                ));

                $backTo = '<a href="' . admin_url('post.php?post=' . $this->parentFormId . '&action=edit') . '">' . esc_html__('Back to Form', 'wordpress-form-wizard') . '</a>';
            }
        }

        $types = array(
            'radio' => esc_html__( 'Radio (One Select)', 'wordpress-form-wizard' ),
            'checkbox' => esc_html__( 'Checkbox (Multi Select)', 'wordpress-form-wizard' ),
            'slider' => esc_html__( 'Slider', 'wordpress-form-wizard' ),
            'textarea' => esc_html__( 'Textarea', 'wordpress-form-wizard' ),
        );

        $columnss = array(
            '2' => esc_html__( '2 Columns', 'wordpress-form-wizard' ),
            '3' => esc_html__( '3 Columns', 'wordpress-form-wizard' ),
            '4' => esc_html__( '4 Columns', 'wordpress-form-wizard' ),
            '6' => esc_html__( '6 Columns', 'wordpress-form-wizard' ),
        );

        $styles = array(
            'image_top' => esc_html__( 'Image Top', 'wordpress-form-wizard' ),
            'image_left' => esc_html__( 'Image Left', 'wordpress-form-wizard' ),
        );

        $yes_no = array(
            'no' => esc_html__('No', 'wordpress-form-wizard'),
            'yes' => esc_html__('Yes', 'wordpress-form-wizard'),
        );

        $or_and = array(
            'OR' => esc_html__('OR', 'wordpress-form-wizard'),
            'AND' => esc_html__('AND', 'wordpress-form-wizard'),
        );

        $filterings = array(
            'filtering' => esc_html__( 'Filtering', 'wordpress-form-wizard' ),
            'ranking' => esc_html__( 'Ranking (nice to have)', 'wordpress-form-wizard' ),
        );        

        echo '<div class="wordpress-form-wizard-form-general">';

            ?>
            <div class="wordpress-form-wizard-row">
                <div class="wordpress-form-wizard-col-xs-3">
                    <?php echo $backTo ?>
                </div>
            </div>
            
            <div class="wordpress-form-wizard-row">
                <div class="wordpress-form-wizard-col-xs-3">
                    <div class="wordpress-form-wizard-badge wordpress-form-wizard-badge-info">
                        <div class="wordpress-form-wizard-badge-title">
                            <?php echo $views ?>
                        </div>
                        <div class="wordpress-form-wizard-badge-text">
                            <?php esc_html_e('Viewed', 'wordpress-form-wizard') ?>
                        </div>
                    </div>
                </div>
                <div class="wordpress-form-wizard-col-xs-3">
                    <div class="wordpress-form-wizard-badge wordpress-form-wizard-badge-primary">
                        <div class="wordpress-form-wizard-badge-title">
                            <?php echo $completed ?>
                        </div>
                        <div class="wordpress-form-wizard-badge-text">
                            <?php esc_html_e('Completed', 'wordpress-form-wizard') ?>
                        </div>
                    </div>
                </div>
                <div class="wordpress-form-wizard-col-xs-3">
                    <div class="wordpress-form-wizard-badge wordpress-form-wizard-badge-alert">
                        <div class="wordpress-form-wizard-badge-title">
                            <?php echo $views - $completed ?>
                        </div>
                        <div class="wordpress-form-wizard-badge-text">
                            <?php esc_html_e('Bounced', 'wordpress-form-wizard') ?>
                        </div>
                    </div>
                </div>
            </div>
            <div class="wordpress-form-wizard-row">

                <div class="wordpress-form-wizard-col-xs-6">
                    <label for="<?php echo $this->prefix ?>internal"><?php esc_html_e( 'Internal Name', 'wordpress-form-wizard' ) ?></label>
                    <input type="text" name="<?php echo $this->prefix ?>internal" class="wordpress-form-wizard-input-field" value="<?php echo $internal ?>">
                </div>

                <div class="wordpress-form-wizard-col-xs-6">
                    <label for="<?php echo $this->prefix ?>breadcrumb"><?php esc_html_e( 'Breadcrumb', 'wordpress-form-wizard' ) ?></label>
                    <input type="text" name="<?php echo $this->prefix ?>breadcrumb" class="wordpress-form-wizard-input-field" value="<?php echo $breadcrumb ?>">
                </div>

                <div class="wordpress-form-wizard-col-xs-12">
                    <label for="<?php echo $this->prefix ?>description"><?php esc_html_e( 'Description', 'wordpress-form-wizard' ) ?></label>
                    <?php
                    echo wp_editor( $description, $this->prefix . 'description', array(
                        'textarea_rows' => 5
                    ) );
                    ?>
                </div>

                <div class="wordpress-form-wizard-col-xs-4">
                    <label for="<?php echo $this->prefix ?>type"><?php esc_html_e( 'Answer Type', 'wordpress-form-wizard' ) ?></label>
                    <select name="<?php echo $this->prefix ?>type" class="wordpress-form-wizard-input-field">
                    <?php
                    foreach ($types as $key => $type_name) {
                        $selected = "";
                        if($type == $key) {
                            $selected = 'selected="selected"';
                        }
                        echo '<option value="' . $key . '" ' . $selected . '>' . $type_name . '</option>';
                    }
                    ?>
                    </select>
                </div>

                <div class="wordpress-form-wizard-col-xs-4">
                    <label for="<?php echo $this->prefix ?>filtering"><?php esc_html_e( 'Answer filtering', 'wordpress-form-wizard' ) ?></label>
                    <select name="<?php echo $this->prefix ?>filtering" class="wordpress-form-wizard-input-field">
                    <?php
                    foreach ($filterings as $key => $filtering_name) {
                        $selected = "";
                        if($filtering == $key) {
                            $selected = 'selected="selected"';
                        }
                        echo '<option value="' . $key . '" ' . $selected . '>' . $filtering_name . '</option>';
                    }
                    ?>
                    </select>
                </div>

                <div class="wordpress-form-wizard-col-xs-4">
                    <label for="<?php echo $this->prefix ?>columns"><?php esc_html_e( 'Answer Columns', 'wordpress-form-wizard' ) ?></label>
                    <select name="<?php echo $this->prefix ?>columns" class="wordpress-form-wizard-input-field">
                    <?php
                    foreach ($columnss as $key => $columns_name) {
                        $selected = "";
                        if($columns == $key) {
                            $selected = 'selected="selected"';
                        }
                        echo '<option value="' . $key . '" ' . $selected . '>' . $columns_name . '</option>';
                    }
                    ?>
                    </select>
                </div>


                <div class="wordpress-form-wizard-col-xs-4">
                    <label for="<?php echo $this->prefix ?>style"><?php esc_html_e( 'Answer Style', 'wordpress-form-wizard' ) ?></label>
                    <select name="<?php echo $this->prefix ?>style" class="wordpress-form-wizard-input-field">
                    <?php
                    foreach ($styles as $key => $style_name) {
                        $selected = "";
                        if($style == $key) {
                            $selected = 'selected="selected"';
                        }
                        echo '<option value="' . $key . '" ' . $selected . '>' . $style_name . '</option>';
                    }
                    ?>
                    </select>
                </div>

                <div class="wordpress-form-wizard-col-xs-4">
                    <label for="<?php echo $this->prefix ?>required"><?php esc_html_e( 'Answer Required', 'wordpress-form-wizard' ) ?></label>
                    <select name="<?php echo $this->prefix ?>required" class="wordpress-form-wizard-input-field">
                    <?php
                    foreach ($yes_no as $key => $required_name) {
                        $selected = "";
                        if($required == $key) {
                            $selected = 'selected="selected"';
                        }
                        echo '<option value="' . $key . '" ' . $selected . '>' . $required_name . '</option>';
                    }
                    ?>
                    </select>
                </div>

                <div class="wordpress-form-wizard-col-xs-4">
                    <label for="<?php echo $this->prefix ?>relation"><?php esc_html_e( 'Relation', 'wordpress-form-wizard' ) ?></label>
                    <select name="<?php echo $this->prefix ?>relation" class="wordpress-form-wizard-input-field">
                    <?php
                    foreach ($or_and as $key => $or_and_name) {
                        $selected = "";
                        if($relation == $key) {
                            $selected = 'selected="selected"';
                        }
                        echo '<option value="' . $key . '" ' . $selected . '>' . $or_and_name . '</option>';
                    }
                    ?>
                    </select>
                </div>
            </div>

            <?php if($type == "slider") { ?>
            <div class="wordpress-form-wizard-row">

                <?php if($type == "price_calculation") { ?>
                <div class="wordpress-form-wizard-col-xs-4">
                    <label for="<?php echo $this->prefix ?>slider_price"><?php esc_html_e( 'Slider Price Per Value', 'wordpress-form-wizard' ) ?></label>
                    <input type="number" step="0.01" name="<?php echo $this->prefix ?>slider_price" value="<?php echo $slider_price ?>">
                </div>     
                <?php } ?>

                <div class="wordpress-form-wizard-col-xs-4">
                    <label for="<?php echo $this->prefix ?>slider_suffix"><?php esc_html_e( 'Slider Suffix', 'wordpress-form-wizard' ) ?></label>
                    <input type="text" name="<?php echo $this->prefix ?>slider_suffix" value="<?php echo $slider_suffix ?>">
                </div>     

                <div class="wordpress-form-wizard-col-xs-4">
                    <label for="<?php echo $this->prefix ?>slider_min"><?php esc_html_e( 'Slider min', 'wordpress-form-wizard' ) ?></label>
                    <input type="number" name="<?php echo $this->prefix ?>slider_min" value="<?php echo $slider_min ?>">
                </div>     
                <div class="wordpress-form-wizard-col-xs-4">
                    <label for="<?php echo $this->prefix ?>slider_max"><?php esc_html_e( 'Slider max', 'wordpress-form-wizard' ) ?></label>
                    <input type="number" name="<?php echo $this->prefix ?>slider_max" value="<?php echo $slider_max ?>">
                </div>      
                <div class="wordpress-form-wizard-col-xs-4">
                    <label for="<?php echo $this->prefix ?>slider_value"><?php esc_html_e( 'Slider Value Default', 'wordpress-form-wizard' ) ?></label>
                    <input type="number" name="<?php echo $this->prefix ?>slider_value" value="<?php echo $slider_value ?>">
                </div>      
                <div class="wordpress-form-wizard-col-xs-4">
                    <label for="<?php echo $this->prefix ?>slider_step"><?php esc_html_e( 'Slider Steps', 'wordpress-form-wizard' ) ?></label>
                    <input type="number" name="<?php echo $this->prefix ?>slider_step" value="<?php echo $slider_step ?>">
                </div>      

            </div>
            <?php } ?>

            <div class="wordpress-form-wizard-row wordpress-form-wizard-depdencies">

                <div class="wordpress-form-wizard-col-xs-12">
                    <strong><?php esc_html_e('Dependencies', 'wordpress-form-wizard') ?></strong>
                </div>

                <?php 
                if(!empty($dependencies)) {
                    foreach($dependencies as $dependencyQuestion => $dependencyData) {
                        $values = $dependencyData['values'];
                        $operator = $dependencyData['operator'];

                        $answers = get_posts(array(
                            'post_type' => 'wizard_answer',
                            'post__in' => $values,
                            'post_status' => 'any',
                            'orderby' => 'post__in'
                        ));

                        $tmp = array();
                        $answersOptions = '<option value="">' . esc_html__('Select a Question First', 'wordpress-form-wizard') . '</option>';
                        if(!empty($answers)) {
                            foreach($answers as $answer) {

                                $selected = "";
                                if(in_array($answer->ID, $values)) {
                                    $selected = 'selected="selected"';
                                }
                                $answersOptions .= '<option value="' . $answer->ID . '" ' . $selected . '>' . $answer->post_title . '</option>';
                            }
                        }
                        $answers = $tmp;

                        ?>
                        <div class="wordpress-form-wizard-col-xs-12 p0 wordpress-form-wizard-depdencies-dependency <?php echo $css ?>">
                            <div class="wordpress-form-wizard-row">
                                <div class="wordpress-form-wizard-col-xs-5">
                                    <select name="<?php echo $this->prefix ?>dependencies[<?php echo $dependencyQuestion ?>][question]" class="wordpress-form-wizard-input-field wordpress-form-wizard-depdencies-dependency-question">
                                    <?php 
                                    echo '<option value="">' . esc_html__('Select a Question', 'wordpress-form-wizard') . '</option>';
                                    foreach ($questions as $question) {

                                        if($question->ID == $post->ID) {
                                            continue;
                                        }
                                        
                                        $selected = "";
                                        if($dependencyQuestion == $question->ID) {
                                            $selected = 'selected="selected"';
                                        }

                                        $dependencyQuestionName = $question->post_title;
                                        $internalName = get_post_meta($question->ID, $this->prefix . 'internal', true);
                                        if(!empty($internalName)) {
                                            $dependencyQuestionName = $internalName;
                                        }

                                        echo '<option value="' . $question->ID . '" ' . $selected . '>' . $dependencyQuestionName . '</option>';
                                    }
                                    ?>
                                    </select>
                                </div>
                                <div class="wordpress-form-wizard-col-xs-2">
                                    <select name="<?php echo $this->prefix ?>dependencies[<?php echo $dependencyQuestion ?>][operator]" class="wordpress-form-wizard-input-field wordpress-form-wizard-depdencies-dependency-operator">
                                        <option value="contains"><?php esc_html_e('Contains', 'wordpress-form-wizard') ?></option>
                                    </select>
                                </div>
                                <div class="wordpress-form-wizard-col-xs-5">
                                    <select name="<?php echo $this->prefix ?>dependencies[<?php echo $dependencyQuestion ?>][values][]" multiple class="wordpress-form-wizard-input-field wordpress-form-wizard-depdencies-dependency-values welaunch-select2">
                                        <?php echo $answersOptions ?>
                                    </select>
                                </div>
                            </div>
                        </div>

                        <?php
                    }
                }
                ?>

                <?php 
                $questionsOptions = '<option value="">' . esc_html__('Select a Question', 'wordpress-form-wizard') . '</option>';
                foreach ($questions as $question) {
                    $selected = "";

                    $dependencyQuestionName = $question->post_title;
                    $internalName = get_post_meta($question->ID, $this->prefix . 'internal', true);
                    if(!empty($internalName)) {
                        $dependencyQuestionName = $internalName;
                    }

                    $questionsOptions .= '<option value="' . $question->ID . '" ' . $selected . '>' . $question->post_title . '</option>';
                }

                for ($i=0; $i < 5; $i++) { 

                    $css = 'wordpress-form-wizard-depdencies-dependency-hidden';
                    if($i == 0 && empty($dependencies)) {
                        $css = '';
                    }
                    ?>

                    <div class="wordpress-form-wizard-col-xs-12 p0 wordpress-form-wizard-depdencies-dependency <?php echo $css ?>">
                        <div class="wordpress-form-wizard-row">
                            <div class="wordpress-form-wizard-col-xs-5">
                                <select name="<?php echo $this->prefix ?>dependencies[<?php echo $i ?>][question]" class="wordpress-form-wizard-input-field wordpress-form-wizard-depdencies-dependency-question">
                                    <?php echo $questionsOptions ?>
                                </select>
                            </div>
                            <div class="wordpress-form-wizard-col-xs-2">
                                <select name="<?php echo $this->prefix ?>dependencies[<?php echo $i ?>][operator]" class="wordpress-form-wizard-input-field wordpress-form-wizard-depdencies-dependency-operator">
                                    <option value="contains"><?php esc_html_e('Contains', 'wordpress-form-wizard') ?></option>
                                </select>
                            </div>
                            <div class="wordpress-form-wizard-col-xs-5">
                                <select name="<?php echo $this->prefix ?>dependencies[<?php echo $i ?>][values][]" multiple class="wordpress-form-wizard-input-field wordpress-form-wizard-depdencies-dependency-values welaunch-select2">
                                    <option value=""><?php esc_html_e('Select a Question First', 'wordpress-form-wizard') ?></option>
                                </select>
                            </div>
                        </div>
                    </div>

                <?php } ?>

                <div class="wordpress-form-wizard-col-xs-12">
                    <div class="wordpress-form-wizard-actions">
                        <a href="#" class="wordpress-form-wizard-actions-action wordpress-form-wizard-depdencies-add-dependency"><i class="bxs bx-add"></i> <?php esc_html_e('Add Depedency', 'wordpress-form-wizard') ?></a>
                    </div>
                </div>

            </div>

        </div>
        <?php

    }


    /**
     * Display Metabox Address
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @return  [type]                       [description]
     */
    public function answers()
    {
        global $post;

        $type = get_post_meta($post->ID, $this->prefix . 'type', true);

        $formType = get_post_meta($this->parentFormId, $this->prefix . 'type', true);
        if($formType == "guided_selling") {

            $possibleTaxonomies = get_taxonomies();
            ksort( $possibleTaxonomies );

            $possibleTaxonomyOperators = array(
                'IN' => 'IN' . ' ' . esc_html__('(contains)', 'wordpress-form-wizard'),
                'NOT IN' => 'NOT IN' . ' ' .  esc_html__('(not contains)', 'wordpress-form-wizard'),
                'EXISTS' => 'EXISTS' . ' ' .  esc_html__('(exists)', 'wordpress-form-wizard'),
                'NOT EXISTS' => 'NOT EXISTS' . ' ' .  esc_html__('(does not exists)', 'wordpress-form-wizard'),
                '>=' => '>=',
                '<=' => '<=',
                '>' => '>',
                '<' => '<',
            );

            $possibleMetaOperators = array(
                '=' => '=',
                '!=' => '!=',
                '>' => '>',
                '>=' => '>=',
                '<' => '<',
                '<=' => '<=',
                'LIKE' => 'LIKE',
                'NOT LIKE' => 'NOT LIKE',
                'IN' => 'IN',
                'NOT IN' => 'NOT IN',
                'BETWEEN' => 'BETWEEN',
                'NOT BETWEEN' => 'NOT BETWEEN',
                'EXISTS' => 'EXISTS',
                'NOT EXISTS' => 'NOT EXISTS',
                'NOT REGEXP ' => 'NOT REGEXP ',
                'RLIKE' => 'RLIKE',
            );

            $possibleMetaTypes = array(
                'NUMERIC' => 'NUMERIC',
                'BINARY' => 'BINARY',
                'CHAR' => 'CHAR',
                'DATE' => 'DATE',
                'DATETIME' => 'DATETIME',
                'DECIMAL' => 'DECIMAL',
                'SIGNED' => 'SIGNED',
                'TIME' => 'TIME',
                'UNSIGNED' => 'UNSIGNED',
            );
        }

        $answers = get_post_meta($post->ID, $this->prefix . 'answers', true);
        if(!empty($answers)) {

            $answers = get_posts( array(
                'numberposts' => -1,
                'post_type' => 'wizard_answer',
                'post__in' => $answers,
                'post_status' => 'any, trash',
                'orderby' => 'post__in'
            ) );

        } else {
            $answers = array();
        }     

        if($type == "slider" ) {

            $sliderTaxonomyName = get_post_meta($post->ID, $this->prefix . 'slider_taxonomy_name', true);
            $sliderTaxonomyOperator = get_post_meta($post->ID, $this->prefix . 'slider_taxonomy_operator', true);
            $sliderMetaKey = get_post_meta($post->ID, $this->prefix . 'slider_meta_key', true);
            $sliderMetaOperator = get_post_meta($post->ID, $this->prefix . 'slider_meta_operator', true);
            $sliderMetaType = get_post_meta($post->ID, $this->prefix . 'slider_meta_type', true);

            ?>
            <div class="wordpress-form-wizard-row">
                <div class="wordpress-form-wizard-col-xs-6">
                    <div class="wordpress-form-wizard-row">
                        <div class="wordpress-form-wizard-col-xs-12">
                            <label for="taxonomies_name">
                                <?php esc_html_e('Taxonomy Matching', 'wordpress-form-wizard') ?>
                                <span class="wordpress-form-wizard-tooltip" wordpress-form-wizard-tooltip="<?php esc_html_e( 'Match answers with Taxonomy Terms to display end results. When using < or > operators, only the first value will be used for comparison.', 'wordpress-form-wizard' ) ?>">
                                <i class="bx bxs-info-circle"></i>
                            </label>
                            <select name="<?php echo $this->prefix ?>slider_taxonomy_name" class="wordpress-form-wizard-input-field wordpress-form-wizard-taxonomies-taxonomy-name welaunch-select2">
                            <?php 
                            echo '<option value="">' . esc_html__('Select a Taxonomy', 'wordpress-form-wizard') . '</option>';
                            foreach ($possibleTaxonomies as $possibleTaxonomy) {
                                $selected = "";
                                if($sliderTaxonomyName == $possibleTaxonomy) {
                                    $selected = 'selected="selected"';
                                }

                                echo '<option value="' . $possibleTaxonomy . '" ' . $selected . '>' . $possibleTaxonomy . '</option>';
                            }
                            ?>
                            </select>
                        </div>
                        <div class="wordpress-form-wizard-col-xs-12">
                            <select name="<?php echo $this->prefix ?>slider_taxonomy_operator" class="wordpress-form-wizard-input-field wordpress-form-wizard-taxonomies-taxonomy-operator">
                            <?php 
                            echo '<option value="">' . esc_html__('Select an Operator', 'wordpress-form-wizard') . '</option>';
                            foreach ($possibleTaxonomyOperators as $possibleTaxonomyOperatorKey => $possibleTaxonomyOperatorValue) {
                                $selected = "";
                                if($sliderTaxonomyOperator == $possibleTaxonomyOperatorKey) {
                                    $selected = 'selected="selected"';
                                }
                                echo '<option value="' . $possibleTaxonomyOperatorKey . '" ' . $selected . '>' . $possibleTaxonomyOperatorValue . '</option>';
                            }
                            ?>
                            </select>
                        </div>
                    </div>
                </div>
                <div class="wordpress-form-wizard-col-xs-6">

                    <div class="wordpress-form-wizard-row">
                        <div class="wordpress-form-wizard-col-xs-12">
                            <label for="taxonomies_name">
                                <?php esc_html_e('Post Meta Matching', 'wordpress-form-wizard') ?>
                                <span class="wordpress-form-wizard-tooltip" wordpress-form-wizard-tooltip="<?php esc_html_e( 'When using BETWEEN or IN operators seperate your values with a comma (10,20).', 'wordpress-form-wizard' ) ?>">
                                <i class="bx bxs-info-circle"></i>
                            </label>
                            <select name="<?php echo $this->prefix ?>slider_meta_key" class="wordpress-form-wizard-input-field wordpress-form-wizard-meta-queries-meta-type">
                            <?php 
                            echo '<option value="">' . esc_html__('Select a Meta Key', 'wordpress-form-wizard') . '</option>';
                            foreach ($this->post_meta_keys as $possibleMetaKey) {
                                
                                $selected = "";
                                if($sliderMetaKey == $possibleMetaKey) {
                                    $selected = 'selected="selected"';
                                }
                                echo '<option value="' . $possibleMetaKey . '" ' . $selected . '>' . $possibleMetaKey . '</option>';
                            }
                            ?>
                            </select>
                        </div>
                        <div class="wordpress-form-wizard-col-xs-12">
                            <select name="<?php echo $this->prefix ?>slider_meta_operator" class="wordpress-form-wizard-input-field wordpress-form-wizard-meta-queries-meta-type">
                            <?php 
                            echo '<option value="">' . esc_html__('Select a Meta Operator', 'wordpress-form-wizard') . '</option>';
                            foreach ($possibleMetaOperators as $possibleMetaOperator) {
                                
                                $selected = "";
                                if($sliderMetaOperator == $possibleMetaOperator) {
                                    $selected = 'selected="selected"';
                                }
                                echo '<option value="' . $possibleMetaOperator . '" ' . $selected . '>' . $possibleMetaOperator . '</option>';
                            }
                            ?>
                            </select>
                        </div>
                        <div class="wordpress-form-wizard-col-xs-12">
                            <select name="<?php echo $this->prefix ?>slider_meta_type" class="wordpress-form-wizard-input-field wordpress-form-wizard-meta-queries-meta-type">
                            <?php 
                            echo '<option value="">' . esc_html__('Select a Meta Type', 'wordpress-form-wizard') . '</option>';
                            foreach ($possibleMetaTypes as $possibleMetaType) {
                                
                                $selected = "";
                                if($sliderMetaType == $possibleMetaType) {
                                    $selected = 'selected="selected"';
                                }
                                echo '<option value="' . $possibleMetaType . '" ' . $selected . '>' . $possibleMetaType . '</option>';
                            }
                            ?>
                            </select>
                        </div>
                    </div>
                </div>
            </div>
            <?php
        }

        if($type == "slider" || $type == "textarea") {
            esc_html_e('Slider or Textarea elements have no answers.', 'wordpress-form-wizard');
            return false;
        }

        $image_src = plugin_dir_url( dirname( __FILE__ ) ) . 'assets/img/placeholder.png';

        ?>

        <h3><?php esc_html_e('Answers', 'wordpress-form-wizard') ?> (<?php echo count($answers) ?>)</h3>

        <div class="wordpress-form-wizard-answers">

            <div class="wordpress-form-wizard-answers-answer wordpress-form-wizard-answers-answer-template" data-index="0" style="display: none;">

                    <input type="hidden" class="wordpress-form-wizard-answers-answer-id" name="<?php echo $this->prefix ?>answer[0][id]" value="">

                    <div class="wordpress-form-wizard-row">
                        <div class="wordpress-form-wizard-col-xs-1 wordpress-form-wizard-sort-container">
                            <i class='bx bx-dots-vertical-rounded'></i>    
                        </div>
                        <div class="wordpress-form-wizard-col-xs-1">
                            <div class="wordpress-form-wizard-upload-image">
                                <img src="<?php echo $image_src; ?>" alt="">
                                <input type="hidden" name="<?php echo $this->prefix ?>answer[0][image]" value="">
                            </div>
                            <a href="#" class="wordpress-form-wizard-remove-image"><?php esc_html_e('Remove image', 'wordpress-form-wizard') ?></a>
                        </div>
                        <div class="wordpress-form-wizard-col-xs-2">
                            <div class="wordpress-form-wizard-row">
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <div class="wordpress-form-wizard-answers-answer-name">
                                        <label for="title"><?php esc_html_e('Answer', 'wordpress-form-wizard') ?></label>
                                        <input type="text" name="<?php echo $this->prefix ?>answer[0][title]" value="">
                                    </div>
                                </div>

                                <?php if($formType == "price_calculation") { ?>
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <div class="wordpress-form-wizard-answers-answer-price">
                                        <label for="price"><?php esc_html_e('Price', 'wordpress-form-wizard') ?></label>
                                        <input type="number" step="0.01" name="<?php echo $this->prefix ?>answer[0][price]" value="">
                                    </div>
                                </div>
                                <?php } ?>

                                <div class="wordpress-form-wizard-col-xs-12">
                                    <div class="wordpress-form-wizard-answers-answer-tooltip">
                                        <label for="tooltip"><?php esc_html_e('Tooltip', 'wordpress-form-wizard') ?></label>
                                        <input type="text" name="<?php echo $this->prefix ?>answer[0][tooltip]" value="">
                                    </div>
                                </div>
                            </div>
                        </div>
                        <div class="wordpress-form-wizard-col-xs-3">
                            <div class="wordpress-form-wizard-answers-answer-description">
                                <label for="description"><?php esc_html_e('Description', 'wordpress-form-wizard') ?></label>
                                <textarea name="<?php echo $this->prefix ?>answer[0][description]" cols="50" rows="8"></textarea>
                            </div>
                        </div>
                        <?php if($formType == "guided_selling") { ?>
                        <div class="wordpress-form-wizard-col-xs-2">

                            <div class="wordpress-form-wizard-row">
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <label for="taxonomies_name">
                                        <?php esc_html_e('Taxonomy Matching', 'wordpress-form-wizard') ?>
                                        <span class="wordpress-form-wizard-tooltip" wordpress-form-wizard-tooltip="<?php esc_html_e( 'Match answers with Taxonomy Terms to display end results. When using < or > operators, only the first value will be used for comparison.', 'wordpress-form-wizard' ) ?>">
                                        <i class="bx bxs-info-circle"></i>
                                    </label>
                                    <select name="<?php echo $this->prefix ?>answer[0][taxonomies][INDEX][name]" class="wordpress-form-wizard-input-field wordpress-form-wizard-taxonomies-taxonomy-name welaunch-select2">
                                    <?php 
                                    echo '<option value="">' . esc_html__('Select a Taxonomy', 'wordpress-form-wizard') . '</option>';
                                    foreach ($possibleTaxonomies as $possibleTaxonomy) {
                                        $selected = "";
                                        echo '<option value="' . $possibleTaxonomy . '" ' . $selected . '>' . $possibleTaxonomy . '</option>';
                                    }
                                    ?>
                                    </select>
                                </div>
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <select name="<?php echo $this->prefix ?>answer[0][taxonomies][INDEX][operator]" class="wordpress-form-wizard-input-field wordpress-form-wizard-taxonomies-taxonomy-operator">
                                    <?php 
                                    echo '<option value="">' . esc_html__('Select an Operator', 'wordpress-form-wizard') . '</option>';
                                    foreach ($possibleTaxonomyOperators as $possibleTaxonomyOperatorKey => $possibleTaxonomyOperatorValue) {
                                        $selected = "";
                                        echo '<option value="' . $possibleTaxonomyOperatorKey . '" ' . $selected . '>' . $possibleTaxonomyOperatorValue . '</option>';
                                    }
                                    ?>
                                    </select>
                                </div>
                                <div class="wordpress-form-wizard-col-xs-12">

                                    <select name="<?php echo $this->prefix ?>answer[0][taxonomies][INDEX][values][]" multiple class="wordpress-form-wizard-input-field wordpress-form-wizard-taxonomies-taxonomy-values welaunch-select2">
                                        <option value=""><?php esc_html_e('Select a Taxonomy first', 'wordpress-form-wizard') ?></option>
                                    </select>
                                </div>
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <label for="posts"><?php esc_html_e('Matching Posts', 'wordpress-form-wizard') ?></label>
                                    <select name="<?php echo $this->prefix ?>answer[0][posts][]" class="wordpress-form-wizard-input-field wordpress-form-wizard-posts" multiple>
                                    <?php 
                                    if(!empty($answer_posts)) {
                                        foreach($answer_posts as $answer_post_id) {
                                            echo '<option value="' . $answer_post_id . '" selected="selected">' . get_the_title($answer_post_id) . '</option>';
                                        }
                                    }  else {
                                        echo '<option value="">' . esc_html__('Select Custom Post', 'wordpress-form-wizard') . '</option>';
                                    }
                                    
                                    ?>
                                    </select>
                                </div>
                            </div>
                        </div>
                        <div class="wordpress-form-wizard-col-xs-2">

                            <div class="wordpress-form-wizard-row">
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <label for="taxonomies_name">
                                        <?php esc_html_e('Post Meta Matching', 'wordpress-form-wizard') ?>
                                        <span class="wordpress-form-wizard-tooltip" wordpress-form-wizard-tooltip="<?php esc_html_e( 'When using BETWEEN or IN operators seperate your values with a comma (10,20).', 'wordpress-form-wizard' ) ?>">
                                        <i class="bx bxs-info-circle"></i>
                                    </label>
                                    <select name="<?php echo $this->prefix ?>answer[0][meta_queries][INDEX][meta_key]" class="wordpress-form-wizard-input-field wordpress-form-wizard-meta-queries-meta-type">
                                    <?php 
                                    echo '<option value="">' . esc_html__('Select a Meta Key', 'wordpress-form-wizard') . '</option>';
                                    foreach ($this->post_meta_keys as $possibleMetaKey) {
                                        
                                        $selected = "";
                                        echo '<option value="' . $possibleMetaKey . '" ' . $selected . '>' . $possibleMetaKey . '</option>';
                                    }
                                    ?>
                                    </select>
                                </div>
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <select name="<?php echo $this->prefix ?>answer[0][meta_queries][INDEX][meta_operator]" class="wordpress-form-wizard-input-field wordpress-form-wizard-meta-queries-meta-type">
                                    <?php 
                                    echo '<option value="">' . esc_html__('Select a Meta Operator', 'wordpress-form-wizard') . '</option>';
                                    foreach ($possibleMetaOperators as $possibleMetaOperator) {
                                        
                                        $selected = "";
                                        echo '<option value="' . $possibleMetaOperator . '" ' . $selected . '>' . $possibleMetaOperator . '</option>';
                                    }
                                    ?>
                                    </select>
                                </div>


                                <div class="wordpress-form-wizard-col-xs-12">
                                    <input type="text" name="<?php echo $this->prefix ?>answer[0][meta_queries][INDEX][meta_value]" class="wordpress-form-wizard-input-field wordpress-form-wizard-meta-queries-meta-value" value="">
                                </div>
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <select name="<?php echo $this->prefix ?>answer[0][meta_queries][INDEX][meta_type]" class="wordpress-form-wizard-input-field wordpress-form-wizard-meta-queries-meta-type">
                                    <?php 
                                    echo '<option value="">' . esc_html__('Select a Meta Type', 'wordpress-form-wizard') . '</option>';
                                    foreach ($possibleMetaTypes as $possibleMetaType) {
                                        
                                        $selected = "";
                                        echo '<option value="' . $possibleMetaType . '" ' . $selected . '>' . $possibleMetaType . '</option>';
                                    }
                                    ?>
                                    </select>
                                </div>
                            </div>
                        </div>
                        <?php } ?>
                        <div class="wordpress-form-wizard-col-xs-1 wordpress-form-wizard-middle-xs d-flex">
                            <div class="wordpress-form-wizard-badge wordpress-form-wizard-badge-info">
                                <div class="wordpress-form-wizard-badge-title">
                                    0
                                </div>
                                <div class="wordpress-form-wizard-badge-text">
                                    <?php esc_html_e('Choosen', 'wordpress-form-wizard') ?>
                                </div>
                            </div>    
                            <div class="wordpress-form-wizard-answers-answer-actions">
                                <a class="wordpress-form-wizard-answers-answer-actions-hide" data-id="" href=""><i class='bx bx-hide'></i></a>
                            </div>
                        </div>
                    </div>  
                </div>

            <?php foreach($answers as $answer) { 

                $price = get_post_meta($answer->ID, $this->prefix . 'price', true);
                $tooltip = get_post_meta($answer->ID, $this->prefix . 'tooltip', true);
                $description = get_post_meta($answer->ID, $this->prefix . 'description', true);
                $image = get_post_meta($answer->ID, $this->prefix . 'image', true);
                $image_src = plugin_dir_url( dirname( __FILE__ ) ) . 'assets/img/placeholder.png';
                if(!empty($image)) {
                    $image_src_data = wp_get_attachment_image_src($image);
                    if(isset($image_src_data[0])) {
                        $image_src = $image_src_data[0];
                    }
                }

                $choosen = get_post_meta($answer->ID, $this->prefix . 'choosen', true);
                if(!$choosen) {
                    $choosen = 0;
                }

                $answer_posts = get_post_meta($answer->ID, $this->prefix . 'posts', true);

                ?>
                <div class="wordpress-form-wizard-answers-answer <?php echo $answer->post_status == "trash" ? 'wordpress-form-wizard-answers-answer-disabled' : '' ?>">

                    <input type="hidden" class="wordpress-form-wizard-answers-answer-id" name="<?php echo $this->prefix ?>answer[<?php echo $answer->ID?>][id]" value="<?php echo $answer->ID ?>">

                    <div class="wordpress-form-wizard-row">
                        <div class="wordpress-form-wizard-col-xs-1 wordpress-form-wizard-sort-container">
                            <i class='bx bx-dots-vertical-rounded'></i>    
                        </div>
                        <div class="wordpress-form-wizard-col-xs-1">
                            <div class="wordpress-form-wizard-upload-image">
                                <img src="<?php echo $image_src; ?>" alt="">
                                <input type="hidden" name="<?php echo $this->prefix ?>answer[<?php echo $answer->ID?>][image]" value="<?php echo $image ?>">
                            </div>
                            <a href="#" class="wordpress-form-wizard-remove-image"><?php esc_html_e('Remove image', 'wordpress-form-wizard') ?></a>
                        </div>
                        <div class="wordpress-form-wizard-col-xs-2">
                            <div class="wordpress-form-wizard-row">
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <div class="wordpress-form-wizard-answers-answer-name">
                                        <label for="title"><?php esc_html_e('Answer', 'wordpress-form-wizard') ?></label>
                                        <input type="text" name="<?php echo $this->prefix ?>answer[<?php echo $answer->ID?>][title]" value="<?php echo $answer->post_title ?>">
                                    </div>
                                </div>

                                <?php if($formType == "price_calculation") { ?>
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <div class="wordpress-form-wizard-answers-answer-price">
                                        <label for="price"><?php esc_html_e('Price', 'wordpress-form-wizard') ?></label>
                                        <input type="number" step="0.01" name="<?php echo $this->prefix ?>answer[<?php echo $answer->ID?>][price]" value="<?php echo $price ?>">
                                    </div>
                                </div>
                                <?php } ?>

                                <div class="wordpress-form-wizard-col-xs-12">
                                    <div class="wordpress-form-wizard-answers-answer-tooltip">
                                        <label for="tooltip"><?php esc_html_e('Tooltip', 'wordpress-form-wizard') ?></label>
                                        <input type="text" name="<?php echo $this->prefix ?>answer[<?php echo $answer->ID?>][tooltip]" value="<?php echo $tooltip ?>">
                                    </div>
                                </div>
                            </div>
                        </div>
                        <div class="wordpress-form-wizard-col-xs-3">
                            <div class="wordpress-form-wizard-answers-answer-description">
                                <label for="description"><?php esc_html_e('Description', 'wordpress-form-wizard') ?></label>
                                <textarea name="<?php echo $this->prefix ?>answer[<?php echo $answer->ID?>][description]" cols="50" rows="8"><?php echo $description ?></textarea>
                            </div>
                        </div>
                        <?php 
                        if($formType == "guided_selling") { 

                            $taxonomies = get_post_meta($answer->ID, $this->prefix . 'taxonomies', true);
                            $taxonomies_name = isset($taxonomies[0]['name']) && !empty($taxonomies[0]['name']) ? $taxonomies[0]['name'] : '';
                            $taxonomies_values = isset($taxonomies[0]['values']) && !empty($taxonomies[0]['values']) ? $taxonomies[0]['values'] : array();
                            $taxonomies_operator = isset($taxonomies[0]['operator']) && !empty($taxonomies[0]['operator']) ? $taxonomies[0]['operator'] : 'IN';
                        ?>
                        <div class="wordpress-form-wizard-col-xs-2">

                            <div class="wordpress-form-wizard-row">
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <label for="taxonomies_name"><?php esc_html_e('Taxonomy Matching', 'wordpress-form-wizard') ?></label>
                                    <select name="<?php echo $this->prefix ?>answer[<?php echo $answer->ID?>][taxonomies][0][name]" class="wordpress-form-wizard-input-field wordpress-form-wizard-taxonomies-taxonomy-name welaunch-select2">
                                    <?php 
                                    echo '<option value="">' . esc_html__('Select a Taxonomy', 'wordpress-form-wizard') . '</option>';
                                    foreach ($possibleTaxonomies as $possibleTaxonomy) {
                                        
                                        $selected = "";
                                        if($taxonomies_name == $possibleTaxonomy) {
                                            $selected = 'selected="selected"';
                                        }

                                        echo '<option value="' . $possibleTaxonomy . '" ' . $selected . '>' . $possibleTaxonomy . '</option>';
                                    }
                                    ?>
                                    </select>
                                </div>
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <select name="<?php echo $this->prefix ?>answer[<?php echo $answer->ID?>][taxonomies][0][operator]" class="wordpress-form-wizard-input-field wordpress-form-wizard-taxonomies-taxonomy-operator">
                                    <?php 
                                    echo '<option value="">' . esc_html__('Select an Operator', 'wordpress-form-wizard') . '</option>';
                                    foreach ($possibleTaxonomyOperators as $possibleTaxonomyOperatorKey => $possibleTaxonomyOperatorValue) {
                                        
                                        $selected = "";
                                        if($taxonomies_operator == $possibleTaxonomyOperatorKey) {
                                            $selected = 'selected="selected"';
                                        }

                                        echo '<option value="' . $possibleTaxonomyOperatorKey . '" ' . $selected . '>' . $possibleTaxonomyOperatorValue . '</option>';
                                    }
                                    ?>
                                    </select>
                                </div>
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <select name="<?php echo $this->prefix ?>answer[<?php echo $answer->ID?>][taxonomies][0][values][]" multiple class="wordpress-form-wizard-input-field wordpress-form-wizard-taxonomies-taxonomy-values welaunch-select2">
                                        <?php 
                                        $possibleTerms = array();
                                        if(isset($taxonomies_name) && !empty($taxonomies_name)) {
                                            $possibleTerms = get_terms( array(
                                                'taxonomy' => $taxonomies_name,
                                                'hide_empty' => false,
                                                'fields' => 'id=>name',
                                            ));
                                        }

                                        foreach ($possibleTerms as $possibleTermId => $possibleTermName) {
                                            
                                            $selected = "";
                                            if(in_array($possibleTermId, $taxonomies_values)) {
                                                $selected = 'selected="selected"';
                                            }

                                            echo '<option value="' . $possibleTermId . '" ' . $selected . '>' . $possibleTermName . '</option>';
                                        }
                                        ?>
                                        <option value=""><?php esc_html_e('Select a Taxonomy first', 'wordpress-form-wizard') ?></option>
                                    </select>
                                </div>
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <label for="posts"><?php esc_html_e('Matching Posts', 'wordpress-form-wizard') ?></label>
                                    <select name="<?php echo $this->prefix ?>answer[<?php echo $answer->ID?>][posts][]" class="wordpress-form-wizard-input-field wordpress-form-wizard-posts" multiple>
                                    <?php 
                                    if(!empty($answer_posts)) {
                                        foreach($answer_posts as $answer_post_id) {
                                            echo '<option value="' . $answer_post_id . '" selected="selected">' . get_the_title($answer_post_id) . '</option>';
                                        }
                                    }  else {
                                        echo '<option value="">' . esc_html__('Select Custom Post', 'wordpress-form-wizard') . '</option>';
                                    }
                                    
                                    ?>
                                    </select>
                                </div>
                            </div>


                        </div>
                        <div class="wordpress-form-wizard-col-xs-2">

                            <?php
                            $meta_queries = get_post_meta($answer->ID, $this->prefix . 'meta_queries', true);
                            // var_dump($meta_queries);
                            $meta_key = isset($meta_queries[0]['meta_key']) && !empty($meta_queries[0]['meta_key']) ? $meta_queries[0]['meta_key'] : '';
                            $meta_operator = isset($meta_queries[0]['meta_operator']) && !empty($meta_queries[0]['meta_operator']) ? $meta_queries[0]['meta_operator'] : '=';
                            $meta_value = isset($meta_queries[0]['meta_value']) && !empty($meta_queries[0]['meta_value']) ? $meta_queries[0]['meta_value'] : '';
                            $meta_type = isset($meta_queries[0]['meta_type']) && !empty($meta_queries[0]['meta_type']) ? $meta_queries[0]['meta_type'] : 'CHAR';
                            ?>

                            <div class="wordpress-form-wizard-row">

                                <div class="wordpress-form-wizard-col-xs-12">
                                    <label for="taxonomies_name"><?php esc_html_e('Post Meta Matching', 'wordpress-form-wizard') ?></label>
                                    <select name="<?php echo $this->prefix ?>answer[<?php echo $answer->ID?>][meta_queries][0][meta_key]" class="wordpress-form-wizard-input-field wordpress-form-wizard-meta-queries-meta-type welaunch-select2">
                                    <?php 
                                    echo '<option value="">' . esc_html__('Select a Meta Key', 'wordpress-form-wizard') . '</option>';
                                    foreach ($this->post_meta_keys as $possibleMetaKey) {
                                        
                                        $selected = "";
                                        if($meta_key == $possibleMetaKey) {
                                            $selected = 'selected="selected"';
                                        }

                                        echo '<option value="' . $possibleMetaKey . '" ' . $selected . '>' . $possibleMetaKey . '</option>';
                                    }
                                    ?>
                                    </select>
                                </div>
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <select name="<?php echo $this->prefix ?>answer[<?php echo $answer->ID?>][meta_queries][0][meta_operator]" class="wordpress-form-wizard-input-field wordpress-form-wizard-meta-queries-meta-type">
                                    <?php 
                                    echo '<option value="">' . esc_html__('Select a Meta Operator', 'wordpress-form-wizard') . '</option>';
                                    foreach ($possibleMetaOperators as $possibleMetaOperatorKey => $possibleMetaOperatorValue) {
                                        
                                        $selected = "";
                                        if($meta_operator == $possibleMetaOperatorKey) {
                                            $selected = 'selected="selected"';
                                        }

                                        echo '<option value="' . $possibleMetaOperatorKey . '" ' . $selected . '>' . $possibleMetaOperatorValue . '</option>';
                                    }
                                    ?>
                                    </select>
                                </div>


                                <div class="wordpress-form-wizard-col-xs-12">
                                    <input type="text" name="<?php echo $this->prefix ?>answer[<?php echo $answer->ID?>][meta_queries][0][meta_value]" class="wordpress-form-wizard-input-field wordpress-form-wizard-meta-queries-meta-value" value="<?php echo $meta_value ?>">
                                </div>
                                <div class="wordpress-form-wizard-col-xs-12">
                                    <select name="<?php echo $this->prefix ?>answer[<?php echo $answer->ID?>][meta_queries][0][meta_type]" class="wordpress-form-wizard-input-field wordpress-form-wizard-meta-queries-meta-type">
                                    <?php 
                                    echo '<option value="">' . esc_html__('Select a Meta Type', 'wordpress-form-wizard') . '</option>';
                                    foreach ($possibleMetaTypes as $possibleMetaType) {
                                        
                                        $selected = "";
                                        if($meta_type == $possibleMetaType) {
                                            $selected = 'selected="selected"';
                                        }

                                        echo '<option value="' . $possibleMetaType . '" ' . $selected . '>' . $possibleMetaType . '</option>';
                                    }
                                    ?>
                                    </select>
                                </div>
                            </div>

                        </div>
                        <?php } ?>
                        <div class="wordpress-form-wizard-col-xs-1 d-flex wordpress-form-wizard-middle-xs">
                            <div class="wordpress-form-wizard-badge wordpress-form-wizard-badge-info">
                                <div class="wordpress-form-wizard-badge-title">
                                    <?php echo $choosen ?>
                                </div>
                                <div class="wordpress-form-wizard-badge-text">
                                    <?php esc_html_e('Choosen', 'wordpress-form-wizard') ?>
                                </div>
                            </div>    
                            <div class="wordpress-form-wizard-answers-answer-actions">

                                <?php if($answer->post_status == "trash") { ?>
                                <a class="wordpress-form-wizard-answers-answer-actions-show" data-id="<?php echo $answer->ID ?>" href="#"><i class='bx bx-show'></i></a>
                                <?php } else { ?>
                                <a class="wordpress-form-wizard-answers-answer-actions-hide" data-id="<?php echo $answer->ID ?>" href="#"><i class='bx bx-hide'></i></a>
                                <?php } ?>
                            </div>
                        </div>
                    </div>  
                </div>
            <?php } ?>

        </div>
        <div class="wordpress-form-wizard-actions">
            <div class="wordpress-form-wizard-row">
                <div class="wordpress-form-wizard-col-xs-12">
                    <a href="#" class="wordpress-form-wizard-actions-action wordpress-form-wizard-actions-add-answer"><?php esc_html_e('Add Answer', 'wordpress-form-wizard') ?></a>
                </div>
            </div>
        </div> 
        <?php
    }

    private function is_new()
    {
        global $pagenow;

        if (!is_admin()) return false;

        return in_array( $pagenow, array( 'post-new.php' ) );
    }

    /**
     * Save Custom Metaboxes
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @param   [type]                       $post_id [description]
     * @param   [type]                       $post    [description]
     * @return  [type]                                [description]
     */
    public function save_custom_metaboxes($post_id, $post)
    {
        global $wordpress_form_wizard_options;

        if($post->post_type !== "wizard_question") {
            return false;
        }

        // Is the user allowed to edit the post or page?
        if (!current_user_can('edit_post', $post->ID)) {
            return $post->ID;
        }

        if ($post->post_type == 'revision') {
            return false;
        }

        if (!isset($_POST['wordpress_form_wizard_meta_nonce']) || !wp_verify_nonce($_POST['wordpress_form_wizard_meta_nonce'], basename(__FILE__))) {
            return false;
        }

        // Add values of $ticket_meta as custom fields
        foreach ($_POST as $postKey => $postData) {

            if(!str_contains( $postKey, 'wordpress_form_wizard_')) {
                continue;
            }

            if($postKey == "wordpress_form_wizard_dependencies") {
                $tmp = array();
                foreach($postData as $dependencyData) {
                    if(!isset($dependencyData['values']) || empty($dependencyData['question'])) {
                        continue;
                    }
                    $tmp[$dependencyData['question']] = array(
                        'operator' => $dependencyData['operator'],
                        'values' => $dependencyData['values']
                    );
                }
                $postData = $tmp;
            }

            if(str_contains( $postKey, 'wordpress_form_wizard_answer')) {
                // var_dump($postData);
                // die();
                foreach($postData as $postDataKey => $postDataValue) {

                    if(!isset($postDataValue['id']) || empty($postDataValue['id'])) {
                        continue;
                    }

                    $answerId = (int) $postDataValue['id'];

                    if(isset($postDataValue['title']) && !empty($postDataValue['title'])) {
                        // place the current post and $new_title into array
                        $post_update = array(
                            'ID'         => $answerId,
                            'post_title' => $postDataValue['title']
                        );

                        wp_update_post( $post_update );
                    }

                    foreach($postDataValue as $postDataValueKey => $postDataValueValue) {

                        if($postDataValueKey == "id" || $postDataValueKey == "title") {
                            continue;
                        }

                        update_post_meta($answerId, $this->prefix . $postDataValueKey, $postDataValueValue);
                    }


                }
            
                continue;
            }

            update_post_meta($post->ID, $postKey, $postData);
        }
    }


}
